@@ -1,5 +1,6 @@
 test_that("accuracy generic function works", {
-   hm <- hybridModel(wineind)
+   inputSeries <- ts(rnorm(9), f = 4)
+   hm <- hybridModel(inputSeries)
    expect_error(accuracy(hm), NA)
    expect_error(accuracy(hm$ets), NA)
    expect_error(accuracy(hm$auto.arima), NA)
@@ -7,5 +8,4 @@ test_that("accuracy generic function works", {
    expect_error(accuracy(hm$stlm), NA)
    expect_error(accuracy(hm$tbats), NA)
    expect_error(accuracy(hm$thetam), NA)
-   
-})
\ No newline at end of file
+})
