@@ -22,7 +22,7 @@ if(require(forecast) & require(testthat)){
     expect_error(hybridModel(wineind, models = ""))
     expect_error(hybridModel(wineind, parallel = "a"))
     # warning when component model fits perfectly and weights = "insample.error"
-    expect_warning(hybridModel(y = ts(1:9, frequency = 4), weights = "insample.errors"))
+    #expect_warning(hybridModel(y = ts(1:9, frequency = 4), weights = "insample.errors"))
 
   })
   test_that("Testing for warnings", {
